<?php

namespace Ulmus\User\Authorize;

use Psr\Http\Message\ServerRequestInterface;
use Ulmus\User\Common\AuthorizeContentTypeEnum;
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Lib\Authorize;

class HeaderAuthentication implements AuthorizeMethodInterface
{
    public function connect(ServerRequestInterface $request, UserInterface $user): bool
    {
        if ( null !== ( $auth = $request->getHeaderLine('Authorization') ) ) {

            list($method, $userPass) = explode(' ', $auth, 2) + [ null, null ];

            switch(strtolower(strtolower($method))) {
                case "basic":
                    return $this->basicMethod($user, $userPass);

                default:
                    throw new \InvalidArgumentException("An authentication method must be provided");
            }
        }

        return false;
    }

    protected function basicMethod(UserInterface $user, string $userPassword) : bool
    {
        if ( false === $decoded = base64_decode($userPassword) ) {
            throw new \RuntimeException("Base64 decoding of given username:password failed");
        }

        list($userName, $password) = explode(':', $decoded) + [ null, null ];

        if ( empty($userName) ) {
            throw new \RuntimeException("A username must be provided");
        }
        elseif ( empty($password) ) {
            throw new \RuntimeException("A password must be provided");
        }

        ( new Authorize($user) )->authenticate([ 'email' => $userName, 'username' => $userName ], $password);

        return $user->isLoaded();
    }

    public function catchRequest(ServerRequestInterface $request) : bool
    {
        foreach(array_merge($request->getHeader('Accept'), $request->getHeader('Content-Type')) as $accept) {
            foreach(explode(',', $accept) as $contentType) {
                if ( AuthorizeContentTypeEnum::tryFrom(strtolower($contentType)) ) {
                    return true;
                }
            }
        }

        return false;
    }

}