<?php

namespace Ulmus\User\Middleware;

use Psr\Http\{
    Message\ResponseInterface,
    Message\ServerRequestInterface,
    Server\MiddlewareInterface,
    Server\RequestHandlerInterface
};
use Ulmus\User\Entity\UserInterface;
use Ulmus\User\Authorize\HeaderAuthentication;

class HeaderAuthenticationMiddleware implements MiddlewareInterface
{
    protected HeaderAuthentication $authenticator;

    public function __construct(
        protected UserInterface $entity,
        protected \Closure $loginFailedResponse,
        HeaderAuthentication $authenticator = null,
    ) {
        $this->authenticator = $authenticator ?: new HeaderAuthentication();
    }

    public function process(ServerRequestInterface $request, RequestHandlerInterface $handler): ResponseInterface
    {
        if ( $this->authenticator->catchRequest($request) ) {
            if ( ! $this->authenticator->connect($request, $this->entity) ) {
                return call_user_func($this->loginFailedResponse, [ 'api.process' => "Auth failed" ]);
            }
        }

        return $handler->handle($request);
    }
}
